@@ -873,7 +873,8 @@ map_pointer<pkgCache::Version> pkgCacheGenerator::NewVersion(pkgCache::VerIterat
    
    // Fill it in
    Ver = pkgCache::VerIterator(Cache,Cache.VerP + Version);
-   Ver->d = AllocateInMap<pkgCache::Version::Extra>();
+   auto d = AllocateInMap<pkgCache::Version::Extra>(); // sequence point so Ver can be moved if needed
+   Ver->d = d;
    if (not Ver.PhasedUpdatePercentage(100))
       abort();
 
