@@ -29,6 +29,10 @@
 #endif
 #include <apt-pkg/string_view.h>
 
+#ifdef APT_COMPILING_APT
+#include <xxhash.h>
+#endif
+
 class FileFd;
 class pkgSourceList;
 class OpProgress;
@@ -40,10 +44,13 @@ class APT_HIDDEN pkgCacheGenerator					/*{{{*/
    APT_HIDDEN map_stringitem_t WriteStringInMap(APT::StringView String) { return WriteStringInMap(String.data(), String.size()); };
    APT_HIDDEN map_stringitem_t WriteStringInMap(const char *String);
    APT_HIDDEN map_stringitem_t WriteStringInMap(const char *String, const unsigned long &Len);
-   APT_HIDDEN map_pointer_t AllocateInMap(const unsigned long &size);
+   APT_HIDDEN uint32_t AllocateInMap(const unsigned long &size);
+   template<typename T> map_pointer<T> AllocateInMap() {
+      return map_pointer<T>{AllocateInMap(sizeof(T))};
+   }
 
    // Dirty hack for public users that do not use C++11 yet
-#if __cplusplus >= 201103L
+#if __cplusplus >= 201103L && defined(APT_COMPILING_APT)
    struct string_pointer {
       const char *data_;
       size_t size;
@@ -60,11 +67,7 @@ class APT_HIDDEN pkgCacheGenerator					/*{{{*/
    };
    struct hash {
       uint32_t operator()(string_pointer const &that) const {
-	 uint32_t Hash = 5381;
-	 const char * const end = that.data() + that.size;
-	 for (const char *I = that.data(); I != end; ++I)
-	    Hash = 33 * Hash + *I;
-	 return Hash;
+	 return XXH3_64bits(that.data(), that.size) & 0xFFFFFFFF;
       }
    };
 
@@ -108,15 +111,15 @@ class APT_HIDDEN pkgCacheGenerator					/*{{{*/
 
    bool NewGroup(pkgCache::GrpIterator &Grp, APT::StringView Name);
    bool NewPackage(pkgCache::PkgIterator &Pkg, APT::StringView Name, APT::StringView Arch);
-   map_pointer_t NewVersion(pkgCache::VerIterator &Ver, APT::StringView const &VerStr,
-			    map_pointer_t const ParentPkg, unsigned short const Hash,
-			    map_pointer_t const Next);
-   map_pointer_t NewDescription(pkgCache::DescIterator &Desc,const std::string &Lang, APT::StringView md5sum,map_stringitem_t const idxmd5str);
+   map_pointer<pkgCache::Version> NewVersion(pkgCache::VerIterator &Ver, APT::StringView const &VerStr,
+			    map_pointer<pkgCache::Package> const ParentPkg, uint32_t Hash,
+			    map_pointer<pkgCache::Version> const Next);
+   map_pointer<pkgCache::Description> NewDescription(pkgCache::DescIterator &Desc,const std::string &Lang, APT::StringView md5sum,map_stringitem_t const idxmd5str);
    bool NewFileVer(pkgCache::VerIterator &Ver,ListParser &List);
    bool NewFileDesc(pkgCache::DescIterator &Desc,ListParser &List);
    bool NewDepends(pkgCache::PkgIterator &Pkg, pkgCache::VerIterator &Ver,
-		   map_pointer_t const Version, uint8_t const Op,
-		   uint8_t const Type, map_pointer_t* &OldDepLast);
+		   map_stringitem_t const Version, uint8_t const Op,
+		   uint8_t const Type, map_pointer<pkgCache::Dependency>* &OldDepLast);
    bool NewProvides(pkgCache::VerIterator &Ver, pkgCache::PkgIterator &Pkg,
 		    map_stringitem_t const ProvidesVersion, uint8_t const Flags);
 
@@ -172,7 +175,7 @@ class APT_HIDDEN pkgCacheListParser
 
    // Some cache items
    pkgCache::VerIterator OldDepVer;
-   map_pointer_t *OldDepLast;
+   map_pointer<pkgCache::Dependency> *OldDepLast;
 
    void * const d;
 
@@ -201,13 +204,13 @@ class APT_HIDDEN pkgCacheListParser
    virtual bool NewVersion(pkgCache::VerIterator &Ver) = 0;
    virtual std::vector<std::string> AvailableDescriptionLanguages() = 0;
    virtual APT::StringView Description_md5() = 0;
-   virtual unsigned short VersionHash() = 0;
+   virtual uint32_t VersionHash() = 0;
    /** compare currently parsed version with given version
     *
     * \param Hash of the currently parsed version
     * \param Ver to compare with
     */
-   virtual bool SameVersion(unsigned short const Hash, pkgCache::VerIterator const &Ver);
+   virtual bool SameVersion(uint32_t Hash, pkgCache::VerIterator const &Ver);
    virtual bool UsePackage(pkgCache::PkgIterator &Pkg,
 			   pkgCache::VerIterator &Ver) = 0;
    virtual map_filesize_t Offset() = 0;
